package com.googlecode.jpattern.gwt.client.communication.direct;

import com.googlecode.jpattern.gwt.client.ApplicationProxy;
import com.googlecode.jpattern.gwt.client.communication.AProxy;
import com.googlecode.jpattern.gwt.client.communication.ICallbackAction;
import com.googlecode.jpattern.gwt.client.logger.ILogger;
import com.googlecode.jpattern.gwt.shared.IWebResult;

/**
 * 
 * @author Francesco Cina'
 *
 * @param <T>
 */
public class PostProxy<T extends IWebResult,Z> extends AProxy<T> {

	private final String url;
	private final Z data;
	private final IServerCallPostAction serverCallPostAction;
	private final Class<T> resultClass;
	private final Class<Z> dataClass;

	public PostProxy(IServerCallPostAction serverCallPostAction, Class<T> resultClass, Class<Z> dataClass, ICallbackAction<T> callbackAction, String url, Z data) {
		super(callbackAction);
		this.serverCallPostAction = serverCallPostAction;
		this.resultClass = resultClass;
		this.dataClass = dataClass;
		this.url=url;
		this.data = data;
	}
	
	@Override
	protected void execute(ICallbackAction<T> callbackAction) throws Exception {
		ILogger logger = ApplicationProxy.getInstance().getApplicationProvider().getLoggerService().getLogger("PostProxy");
		
		logger.info("Begin execute - call URL " + url);

		try {
			callbackAction.onSuccess( serverCallPostAction.post(resultClass, dataClass, url, data) );
		} catch (Exception e) {
			callbackAction.onError(e);
		}
	}

}
