package com.googlecode.jpattern.gwt.client.communication.rest;

import com.google.gwt.http.client.Request;
import com.google.gwt.http.client.RequestCallback;
import com.google.gwt.http.client.Response;
import com.googlecode.jpattern.gwt.client.ApplicationProxy;
import com.googlecode.jpattern.gwt.client.IBeanFactory;
import com.googlecode.jpattern.gwt.client.communication.ICallbackAction;
import com.googlecode.jpattern.gwt.client.logger.ILogger;
import com.googlecode.jpattern.gwt.shared.IWebResult;

/**
 * 
 * @author Francesco Cina'
 *
 */
public class ProxyRequestCallback<T extends IWebResult> implements RequestCallback {

	private final ICallbackAction<T> callbackAction;
	private final IObjectSerializer<T> jsonBuilder;
	private final ILogger logger;
	private final IBeanFactory beanFactory;

	public ProxyRequestCallback(IBeanFactory beanFactory, ICallbackAction<T> callbackAction, IObjectSerializer<T> jsonBuilder) {
		this.beanFactory = beanFactory;
		this.callbackAction = callbackAction;
		this.jsonBuilder = jsonBuilder;
		logger = ApplicationProxy.getInstance().getApplicationProvider().getLoggerService().getLogger("RequestCallBackAction");
	}
	
	@Override
	public void onError(Request request, Throwable exception) {
		logger.error("RequestCallback error", exception);
		callbackAction.onError(exception);
	}

	@Override
	public void onResponseReceived(Request request, Response response) {
		logger.debug("RequestCallBack OK. StatusCode: [" + response.getStatusCode() + "] StatusText: [" + response.getStatusText() + "]");
		T result;
		try {
			result = jsonBuilder.deserialize(beanFactory, response.getText());
			if (result!=null) {
				callbackAction.onSuccess( result ); 
			} else {
				callbackAction.onError(new Exception("Server call result is null"));
			}
		} catch (Exception e) {
			logger.error("RequestCallback parsing JSON error", e);
		}
	}
}
