package com.googlecode.jpattern.gwt.client.communication.rest;

import com.google.gwt.http.client.RequestBuilder;
import com.googlecode.jpattern.gwt.client.ApplicationProxy;
import com.googlecode.jpattern.gwt.client.IBeanFactory;
import com.googlecode.jpattern.gwt.client.communication.AProxy;
import com.googlecode.jpattern.gwt.client.communication.ICallbackAction;
import com.googlecode.jpattern.gwt.client.logger.ILogger;
import com.googlecode.jpattern.gwt.shared.IWebResult;

/**
 * 
 * @author Francesco Cina'
 *
 * @param <T>
 */
public class PutProxy<T extends IWebResult,Z> extends AProxy<T> {

	private final String url;
	private final Z data;
	private final IObjectSerializer<Z> sendJsonBuilder;
	private final IBeanFactory beanFactory;
	private final IObjectSerializer<T> resultJsonBuilder;

	public PutProxy(IBeanFactory beanFactory, ICallbackAction<T> callbackAction, IObjectSerializer<T> resultJsonBuilder, String url, IObjectSerializer<Z> sendJsonBuilder, Z data) {
		super(callbackAction);
		this.beanFactory = beanFactory;
		this.resultJsonBuilder = resultJsonBuilder;
		this.url=url;
		this.sendJsonBuilder = sendJsonBuilder;
		this.data = data;
	}
	
	@Override
	protected void execute(ICallbackAction<T> callbackAction) throws Exception {
		ILogger logger = ApplicationProxy.getInstance().getApplicationProvider().getLoggerService().getLogger("PutProxy");
		
		logger.info("Begin execute - call URL " + url);

		// Send request to server and catch any errors.
		RequestBuilder builder = new RequestBuilder(RequestBuilder.PUT, url);
		builder.setHeader("Content-type", "application/json");
		
		String json = sendJsonBuilder.serialize(beanFactory, data);
		ProxyRequestCallback<T> requestCallBack = new ProxyRequestCallback<T>(beanFactory, callbackAction, resultJsonBuilder);
		builder.sendRequest(json, requestCallBack);
		logger.debug("End execute - call URL " + url);
	}

}
